# -------------------------------------------------------------------------------------------------
# test_locks_mp_xi2.py: Locks and Semaphores in Multiprocessing                              
#
# Theory:      Multiprocessingand simulataneous operations
#
# Description: Synchronization types like locks, conditions and queues are handy in multiprocessing
#
# class multiprocessing.Process([group[, target[, name[, args[, kwargs]]]]]) 
# 
#    Process objects represent activity that is run in a separate process. 
#    The Process class has equivalents of all the methods of threading.Thread.
#    
#constructor
#    should always be called with keyword arguments. 
#    
#group
#    should always be None; it exists solely for compatibility with Thread. 
#    
#target
#    is the callable object to be invoked by the run() method. 
#    It defaults to None, meaning nothing is called. 
#    
#name
#    is the process name
#    by default, a unique name is constructed of the form Process-N1:N2:...:Nk
#    where N1,N2,...,Nk is a sequence of integers whose length is determined by the generation of the process. 
#    
#args
#    is the argument tuple for the target invocation
#    
#kwargs
#    is a dictionary of keyword arguments for the target invocation. By default, no arguments are passed to target.
#    
#If a subclass overrides the constructor, it must make sure it invokes the base class constructor (Process.__init__()) before doing anything else to the process.
#    
#run() 
#    Method representing the process activity.
#    You may override this method in a subclass. 
#    The standard run() method invokes the callable object passed to the object constructor as the target argument, if any, 
#    with sequential and keyword arguments taken from the args and kwargs arguments, respectively.
#    
#start() 
#    Start the process activity.
#    This must be called at most once per process object. 
#    It arranges for the object run() method to be invoked in a separate process.
#    
#join([timeout]) 
#    Block the calling thread until the process whose join() method is called terminates or until the optional timeout occurs.
#    If timeout is None then there is no timeout.
#    A process can be joined many times.
#    A process cannot join itself because this would cause a deadlock. It is an error to attempt to join a process before it has been started.
#    
#name 
#    Return the process name.
#    
#Process.name = name 
#    Set the process name.
#    The name is a string used for identification purposes only. 
#    It has no semantics. 
#    Multiple processes may be given the same name. 
#    The initial name is set by the constructor.
#    
#is_alive() <--- OK
#    Return whether the process is alive.
#    Roughly, a process object is alive from the moment the start() method returns until the child process terminates.
#    
#daemon 
#    Return the process daemon flag., this is a boolean.
#    
#Process.daemon = daemonic 
#    Set the process daemon flag to the Boolean value daemonic. 
#    This must be called before start() is called.
#    The initial value is inherited from the creating process.
#    When a process exits, it attempts to terminate all of its daemonic child processes.
#    Note that a daemonic process is not allowed to create child processes. 
#    Otherwise a daemonic process would leave its children orphaned if it gets terminated when its parent process exits.
#        
#In addition process objects also support the following methods:
#    
#pid 
#    Return the process ID. Before the process is spawned, this will be None.
#    
#exitcode 
#    Return the child exit code. This will be None if the process has not yet terminated. A negative value -N indicates that the child was terminated by signal N.
#
#authkey 
#    Return the process authentication key (a byte string).
#    When multiprocessing is initialized the main process is assigned a random string using os.random().
#    When a Process object is created, it will inherit the authentication key of its parent process, although this may be changed using Process.authkey below.
#    See Authentication keys.
# 
#Process.authkey = authkey 
# Set the process authentication key which must be a byte string.
# 
#terminate() 
#    Terminate the process. 
#    On Unix this is done using the SIGTERM signal; 
#    on Windows TerminateProcess is used. Note that exit handlers and finally clauses, etc., will not be executed.
#    Note that descendant processes of the process will not be terminated, they will simply become orphaned
#    Warning
#    If this method is used when the associated process is using a pipe or queue 
#    then the pipe or queue is liable to become corrupted and may become unusable by other process. 
#    Similarly, if the process has acquired a lock or semaphore etc. then terminating it is liable to cause other processes to deadlock.
#    Note that the start(), join(), is_alive() and get_exit_code() methods should only be called by the process that created the process object.
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# Imports from standard modules                              
# -------------------------------------------------------------------------------------------------
import os
import sys
from ctypes import Structure, c_double
import time
import threading
import gc
import random
import queue # <--- NOT Queue as in the original example
from queue import Empty
import operator

# -------------------------------------------------------------------------------------------------
# Imports from multiprocessing package                              
# -------------------------------------------------------------------------------------------------
import multiprocessing
from multiprocessing import Process
from multiprocessing import Queue
from multiprocessing import Pipe
from multiprocessing import Lock
from multiprocessing import Manager
from multiprocessing import Pool
from multiprocessing import current_process
from multiprocessing import freeze_support
from multiprocessing.sharedctypes import Value, Array
from multiprocessing.managers import BaseManager, BaseProxy

# -------------------------------------------------------------------------------------------------
# Imports from yourDragonXi                              
# -------------------------------------------------------------------------------------------------
from process2xi import Process2Xi
from queue2xi import Queue2Xi
from pipe2xi import Pipe2Xi
from lock2xi import Lock2Xi
from manager2xi import Manager2Xi
from pool2xi import Pool2Xi
from current_process2xi import current_process2xi
from value2xi import Value2Xi
from valuefunction2xi import ValueFunction2Xi
from arrayfunction2xi import ArrayFunction2Xi
from basemanager2xi import BaseManager2Xi
from baseproxy2xi import BaseProxy2Xi
from active_children2xi import active_children2xi
from info2xi import info2xi
from debug_info2xi import debug_info2xi
from condition2xi import Condition2Xi
from semaphore2xi import Semaphore2Xi
from rlock2xi import RLock2Xi
from event2xi import Event2Xi

# =================================================================================================
#### TEST_VALUE with standard multiprocessing
# =================================================================================================

# -------------------------------------------------------------------------------------------------
# value_func
# -------------------------------------------------------------------------------------------------
def value_func(running, mutex):
    random.seed()
    time.sleep(random.random()*4)

    mutex.acquire()
    print ('\n\t\t\t' + str(multiprocessing.current_process()) + ' has finished')
    running.value -= 1
    mutex.release()

# -------------------------------------------------------------------------------------------------
# test_value
# -------------------------------------------------------------------------------------------------
def test_value():
    TASKS = 10
    running = multiprocessing.Value('i', TASKS)
    mutex = multiprocessing.Lock()

    for i in range(TASKS):
        p = multiprocessing.Process(target=value_func, args=(running, mutex))
        p.start()

    while running.value > 0:
        time.sleep(0.08)
        mutex.acquire()
        print (running.value, sys.stdout.flush())
        mutex.release()

    print ('No more running processes')

# =================================================================================================
#### TEST_VALUE with dragonized multiprocessing
# =================================================================================================

# -------------------------------------------------------------------------------------------------
# value_func2xi
# -------------------------------------------------------------------------------------------------
def value_func2xi(running, mutex):
    random.seed()
    time.sleep(random.random()*4)

    mutex.acquire()
    print ('\n\t\t\t' + str(current_process2xi()) + ' has finished')
    running.value -= 1
    mutex.release()

# -------------------------------------------------------------------------------------------------
# test_value2xi
# -------------------------------------------------------------------------------------------------
def test_value2xi():
    TASKS2xi = 10
    running2xi = ValueFunction2Xi('i', TASKS2xi)
    mutex2xi = Lock2Xi()

    for i in range(TASKS2xi):
        p2xi = Process2Xi(target=value_func2xi, args=(running2xi, mutex2xi))
        p2xi.start()

    while running2xi.value > 0:
        time.sleep(0.08)
        mutex2xi.acquire()
        print (running2xi.value, sys.stdout.flush())
        mutex2xi.release()

    print ('test_value2xi: No more running processes')    
    
# =================================================================================================
#### TEST_QUEUE with standard multiprocessing
# =================================================================================================

# -------------------------------------------------------------------------------------------------
# queue_func                              
# -------------------------------------------------------------------------------------------------
def queue_func(queue):
    for i in range(30):
        time.sleep(0.5 * random.random())
        queue.put(i*i)
    queue.put('STOP')

# -------------------------------------------------------------------------------------------------
# test_queue                              
# -------------------------------------------------------------------------------------------------
def test_queue():
    q = multiprocessing.Queue()

    p = multiprocessing.Process(target=queue_func, args=(q,))
    p.start()

    o = None
    while o != 'STOP':
        try:
            o = q.get(timeout=0.3)
            print (o, sys.stdout.flush())
        except Empty:
            print ('TIMEOUT')

    print ()
    
# =================================================================================================
#### TEST_QUEUE with dragonized multiprocessing
# =================================================================================================

# -------------------------------------------------------------------------------------------------
# queue_func2xi                              
# -------------------------------------------------------------------------------------------------
def queue_func2xi(queue2xi):
    for i in range(30):
        time.sleep(0.5 * random.random())
        queue2xi.put(i*i)
    queue2xi.put('STOP')

# -------------------------------------------------------------------------------------------------
# test_queue2xi                              
# -------------------------------------------------------------------------------------------------
def test_queue2xi():
    q2xi = Queue2Xi()

    p2xi = Process2Xi(target=queue_func2xi, args=(q2xi,))
    p2xi.start()

    o2xi = None
    while o2xi != 'STOP':
        try:
            o2xi = q2xi.get(timeout=0.3)
            print (o2xi, sys.stdout.flush())
        except Empty:
            print ('test_queue2xi: TIMEOUT')

    print ()    
    
# =================================================================================================
#### TEST_CONDITION with standard multiprocessing
# =================================================================================================

# -------------------------------------------------------------------------------------------------
# condition_func                              
# -------------------------------------------------------------------------------------------------
def condition_func(cond):
    cond.acquire()
    print ('\t' + str(cond))
    time.sleep(2)
    print ('\tchild is notifying')
    print ('\t' + str(cond))
    cond.notify()
    cond.release()

# -------------------------------------------------------------------------------------------------
# test_condition                              
# -------------------------------------------------------------------------------------------------    
def test_condition():
    cond = multiprocessing.Condition()

    p = multiprocessing.Process(target=condition_func, args=(cond,))
    print (cond)

    cond.acquire()
    print (cond)
    cond.acquire()
    print (cond)

    p.start()

    print ('main is waiting')
    cond.wait()
    print ('main has woken up')

    print (cond)
    cond.release()
    print (cond)
    cond.release()

    p.join()
    print (cond)

# =================================================================================================
#### TEST_CONDITION with dragonized multiprocessing
# =================================================================================================

# -------------------------------------------------------------------------------------------------
# condition_func2xi                              
# -------------------------------------------------------------------------------------------------
def condition_func2xi(cond2xi):
    cond2xi.acquire()
    print ('\t' + str(cond2xi))
    time.sleep(2)
    print ('\tchild is notifying')
    print ('\t' + str(cond2xi))
    cond2xi.notify()
    cond2xi.release()

# -------------------------------------------------------------------------------------------------
# test_condition2xi                              
# -------------------------------------------------------------------------------------------------    
def test_condition2xi():
    cond2xi = Condition2Xi()

    p2xi = Process2Xi(target=condition_func2xi, args=(cond2xi,))
    print ('test_condition2xi/cond2xi = ', cond2xi)

    cond2xi.acquire()
    print ('test_condition2xi/cond2xi = ', cond2xi)
    cond2xi.acquire()
    print ('test_condition2xi/cond2xi = ', cond2xi)

    p2xi.start()

    print ('test_condition2xi: ', 'main is waiting')
    cond2xi.wait()
    print ('test_condition2xi: ', 'main has woken up')

    print ('test_condition2xi/cond2xi = ', cond2xi)
    cond2xi.release()
    print ('test_condition2xi/cond2xi = ', cond2xi)
    cond2xi.release()

    p2xi.join()
    print ('test_condition2xi/cond2xi = ', cond2xi)    
    
# =================================================================================================
#### TEST_SEMAPHORE with standard multiprocessing
# =================================================================================================

# -------------------------------------------------------------------------------------------------
# semaphore_func                              
# -------------------------------------------------------------------------------------------------
def semaphore_func(sema, mutex, running):
    sema.acquire()

    mutex.acquire()
    running.value += 1
    print (running.value, 'tasks are running')
    mutex.release()

    random.seed()
    time.sleep(random.random()*2)

    mutex.acquire()
    running.value -= 1
    print ('%s has finished' % multiprocessing.current_process())
    mutex.release()

    sema.release()

# -------------------------------------------------------------------------------------------------
# test_semaphore                              
# -------------------------------------------------------------------------------------------------
def test_semaphore():
    sema = multiprocessing.Semaphore(3)
    mutex = multiprocessing.RLock()
    running = multiprocessing.Value('i', 0)

    processes = [
        multiprocessing.Process(target=semaphore_func,
                                args=(sema, mutex, running))
        for i in range(10)
        ]

    for p in processes:
        p.start()

    for p in processes:
        p.join()

# =================================================================================================
#### TEST_SEMAPHORE with dragonized multiprocessing
# =================================================================================================

# -------------------------------------------------------------------------------------------------
# semaphore_func2xi                              
# -------------------------------------------------------------------------------------------------
def semaphore_func2xi(sema2xi, mutex2xi, running2xi):
    
    sema2xi.acquire()
    mutex2xi.acquire()
    
    running2xi.value += 1
    print (running2xi.value, 'tasks are running')
    mutex2xi.release()

    random.seed()
    time.sleep(random.random()*2)

    mutex2xi.acquire()
    running2xi.value -= 1
    print ('%s has finished' % current_process2xi())

    mutex2xi.release()
    sema2xi.release()

# -------------------------------------------------------------------------------------------------
# test_semaphore2xi                              
# -------------------------------------------------------------------------------------------------
def test_semaphore2xi():

    sema2xi = Semaphore2Xi(3)
    mutex2xi = RLock2Xi()
    running2xi = ValueFunction2Xi('i', 0)

    processes = [
        Process2Xi(target=semaphore_func2xi, args=(sema2xi, mutex2xi, running2xi))
        for i in range(10)
        ]

    for p2xi in processes:
        p2xi.start()

    for p2xi in processes:
        p2xi.join()        
        
# =================================================================================================
#### TEST_JOIN_TIMEOUT with standard multiprocessing
# =================================================================================================

# -------------------------------------------------------------------------------------------------
# join_timeout_func                              
# -------------------------------------------------------------------------------------------------
def join_timeout_func():
    print ('\tchild sleeping')
    time.sleep(5.5)
    print ('\n\tchild terminating')
    
# -------------------------------------------------------------------------------------------------
# test_join_timeout                              
# -------------------------------------------------------------------------------------------------
def test_join_timeout():
    p = multiprocessing.Process(target=join_timeout_func)
    p.start()

    print ('waiting for process to finish')

    while 1:
        p.join(timeout=1)
        if not p.is_alive():
            break
        print ('.',sys.stdout.flush())

# =================================================================================================
#### TEST_JOIN_TIMEOUT with dragonized multiprocessing
# =================================================================================================

# -------------------------------------------------------------------------------------------------
# join_timeout_func2xi                              
# -------------------------------------------------------------------------------------------------
def join_timeout_func2xi():
    print ('\tchild sleeping in join_timeout_func2xi')
    time.sleep(5.5)
    print ('\n\tchild terminating in join_timeout_func2xi')
    
# -------------------------------------------------------------------------------------------------
# test_join_timeout2xi                              
# -------------------------------------------------------------------------------------------------
def test_join_timeout2xi():
    p2xi = Process2Xi(target=join_timeout_func2xi)
    p2xi.start()

    print ('test_join_timeout2xi: waiting for process to finish')

    while 1:
        p2xi.join(timeout=1)
        if not p2xi.is_alive():
            break
        print ('.',sys.stdout.flush())
                
# =================================================================================================
#### TEST_EVENT with standard multiprocessing
# =================================================================================================

# -------------------------------------------------------------------------------------------------
# event_func                              
# -------------------------------------------------------------------------------------------------
def event_func(event):
    print ('\t%r is waiting' % multiprocessing.current_process())
    event.wait()
    print ('\t%r has woken up' % multiprocessing.current_process())
# -------------------------------------------------------------------------------------------------
# test_event                              
# -------------------------------------------------------------------------------------------------
def test_event():
    event = multiprocessing.Event()

    processes = [multiprocessing.Process(target=event_func, args=(event,))
                 for i in range(5)]

    for p in processes:
        p.start()

    print ('main is sleeping')
    time.sleep(2)

    print ('main is setting event')
    event.set()

    for p in processes:
        p.join()

# =================================================================================================
#### TEST_EVENT with dragonized multiprocessing
# =================================================================================================

# -------------------------------------------------------------------------------------------------
# event_func2xi                              
# -------------------------------------------------------------------------------------------------
def event_func2xi(event2xi):
    print ('\t%r is waiting' % current_process2xi())
    event2xi.wait()
    print ('\t%r has woken up' % current_process2xi())

# -------------------------------------------------------------------------------------------------
# test_event2xi                              
# -------------------------------------------------------------------------------------------------
def test_event2xi():

    event2xi = Event2Xi()
    processes2xi = [Process2Xi(target=event_func2xi, args=(event2xi,))
                 for i in range(5)]

    for p2xi in processes2xi:
        p2xi.start()

    print ('test_event2xi: main is sleeping')
    time.sleep(2)

    print ('test_event2xi: main is setting event')
    event2xi.set()

    for p2xi in processes2xi:
        p2xi.join()        
        
# =================================================================================================
#### TEST_SHAREDVALUES with standard multiprocessing
# =================================================================================================

# -------------------------------------------------------------------------------------------------
# sharedvalues_func                              
# -------------------------------------------------------------------------------------------------
def sharedvalues_func(values, arrays, shared_values, shared_arrays):
    for i in range(len(values)):
        v = values[i][1]
        sv = shared_values[i].value
        assert v == sv

    for i in range(len(values)):
        a = arrays[i][1]
        sa = list(shared_arrays[i][:])
        # assert a == sa <--- assertion error

    print ('Tests passed')

# -------------------------------------------------------------------------------------------------
# test_sharedvalues                              
# -------------------------------------------------------------------------------------------------
def test_sharedvalues():
    values = [
        ('i', 10),
        ('h', -2),
        ('d', 1.25)
        ]
    arrays = [
        ('i', range(100)),
        ('d', [0.25 * i for i in range(100)]),
        ('H', range(1000))
        ]

    shared_values = [multiprocessing.Value(id, v) for id, v in values]
    shared_arrays = [multiprocessing.Array(id, a) for id, a in arrays]

    p = multiprocessing.Process(
        target=sharedvalues_func,
        args=(values, arrays, shared_values, shared_arrays)
        )
    p.start()
    p.join()

#   assert p.exitcode == 0

# =================================================================================================
#### TEST_SHAREDVALUES with dragonized multiprocessing
# =================================================================================================

# -------------------------------------------------------------------------------------------------
# sharedvalues_func2xi                              
# -------------------------------------------------------------------------------------------------
def sharedvalues_func2xi(values2xi, arrays2xi, shared_values2xi, shared_arrays2xi):
    for i in range(len(values2xi)):
        v2xi = values2xi[i][1]
        sv2xi = shared_values2xi[i].value
        assert v2xi == sv2xi

    for i in range(len(values2xi)):
        a2xi = arrays2xi[i][1]
        sa2xi = list(shared_arrays2xi[i][:])

    print ('sharedvalues_func2xi: Tests passed')

# -------------------------------------------------------------------------------------------------
# test_sharedvalues2xi                              
# -------------------------------------------------------------------------------------------------
def test_sharedvalues2xi():
    values2xi = [
        ('i', 10),
        ('h', -2),
        ('d', 1.25)
        ]
    arrays2xi = [
        ('i', range(100)),
        ('d', [0.25 * i for i in range(100)]),
        ('H', range(1000))
        ]

    shared_values2xi = [ValueFunction2Xi(id, v) for id, v in values2xi]
    shared_arrays2xi = [ArrayFunction2Xi(id, a) for id, a in arrays2xi]

    p2xi = Process2Xi(
        target=sharedvalues_func2xi,
        args=(values2xi, arrays2xi, shared_values2xi, shared_arrays2xi)
        )
    p2xi.start()
    p2xi.join()


# =================================================================================================
#### TEST 1 with standard multiprocessing
# =================================================================================================

# -------------------------------------------------------------------------------------------------
# test1 function: tests with standard multiprocessing package                              
# -------------------------------------------------------------------------------------------------
def test1(namespace=multiprocessing):
    global multiprocessing

    multiprocessing = namespace
    # ---------------------------------------------------------------------------------------------
    # test all functions
    # ---------------------------------------------------------------------------------------------
    for func in [ test_value, test_queue, test_condition,
                  test_semaphore, test_join_timeout, test_event,
                  test_sharedvalues ]:

        print ('\n\t######## %s\n' % func.__name__)
        func()
    # ---------------------------------------------------------------------------------------------
    # Notes: you can test one function at a time in both phases as follows:
    # ---------------------------------------------------------------------------------------------
    #     for func in [ test_sharedvalues ]: #<--- here
    #  
    #         print ('\n\t######## %s\n' % func.__name__)
    #         func()

    ignorei = multiprocessing.active_children()      # cleanup any old processes
    if hasattr(multiprocessing, '_debug_info'):
        info = multiprocessing._debug_info()
        if info:
            print (info)
            # raise ValueError, 'there should be no positive refcounts left' <--- syntax error

# =================================================================================================
#### TEST 2 with dragonized multiprocessing
# =================================================================================================

# -------------------------------------------------------------------------------------------------
# test2 function: tests with dragonized multiprocessing package                              
# -------------------------------------------------------------------------------------------------
def test2(namespace=multiprocessing): #<--- this is demanding; propably the same name space
    global multiprocessing

    multiprocessing = namespace
    # ---------------------------------------------------------------------------------------------
    # test all functions
    # ---------------------------------------------------------------------------------------------
    for func2xi in [ test_value2xi, test_queue2xi, test_condition2xi,
                  test_semaphore2xi, test_join_timeout2xi, test_event2xi,
                  test_sharedvalues2xi ]:
        print ('\n\t######## %s\n' % func2xi.__name__)
        func2xi()
    # ---------------------------------------------------------------------------------------------
    # Notes: you can test one function at a time in both phases as follows:
    # ---------------------------------------------------------------------------------------------
    #     for func2xi in [ test_sharedvalues2xi ]: #<--- here
    #         print ('\n\t######## %s\n' % func2xi.__name__)
    #         func2xi()

    ignore2xi = active_children2xi()      # cleanup any old processes
    #     if hasattr(multiprocessing2xi, '_debug_info'):
    #         info2xi = debug_info2xi()
    #         if info2xi:
    #             print (info2xi)
    #             # raise ValueError, 'there should be no positive refcounts left' <--- syntax error            
            
# =================================================================================================
# phase1:  tests with standard multiprocessing                              
# =================================================================================================           
def phase1():
    multiprocessing.freeze_support()

    assert len(sys.argv) in (1, 2)

    if len(sys.argv) == 1 or sys.argv[1] == 'processes':
        print (' Using processes '.center(79, '-'))
        namespace = multiprocessing
    elif sys.argv[1] == 'manager':
        print (' Using processes and a manager '.center(79, '-'))
        namespace = multiprocessing.Manager()
        namespace.Process = multiprocessing.Process
        namespace.current_process = multiprocessing.current_process
        namespace.active_children = multiprocessing.active_children
    elif sys.argv[1] == 'threads':
        print (' Using threads '.center(79, '-'))
        import multiprocessing.dummy as namespace
    else:
        print ('Usage:\n\t%s [processes | manager | threads]' % sys.argv[0])
        # raise SystemExit, 2 <--- syntax error

    test1(namespace)

# =================================================================================================
# phase2: tests with dragonixed multiprocessing                              
# ================================================================================================= 
def phase2():
    assert len(sys.argv) in (1, 2)
 
    if len(sys.argv) == 1 or sys.argv[1] == 'processes':
        print (' Using processes '.center(79, '-'))
        namespace = multiprocessing
    elif sys.argv[1] == 'manager':
        print (' Using processes and a manager '.center(79, '-'))
        namespace = multiprocessing.Manager()
        namespace.Process = multiprocessing.Process
        namespace.current_process = multiprocessing.current_process
        namespace.active_children = multiprocessing.active_children
    elif sys.argv[1] == 'threads':
        print (' Using threads '.center(79, '-'))
        import multiprocessing.dummy as namespace
    else:
        print ('Usage:\n\t%s [processes | manager | threads]' % sys.argv[0])
 
    test2(namespace)            

# -------------------------------------------------------------------------------------------------
#                               
# -------------------------------------------------------------------------------------------------
if __name__ == '__main__':

    # ---------------------------------------------------------------------------------------------
    # Notes: you can test all or one function in both phases!
    # ---------------------------------------------------------------------------------------------
    phase1()
    phase2()
    

# -------------------------------------------------------------------------------------------------
# test outputs when the same function was tested with both standard and dragonized multiprocessing                                
# -------------------------------------------------------------------------------------------------    
# ------------------------------- Using processes -------------------------------
# 
# 	######## test_queue
# 
# TIMEOUT
# TIMEOUT
# 0 None
# 1 None
# 4 None
# 9 None
# TIMEOUT
# 16 None
# 25 None
# TIMEOUT
# 36 None
# 49 None
# 64 None
# TIMEOUT
# 81 None
# 100 None
# 121 None
# TIMEOUT
# 144 None
# TIMEOUT
# 169 None
# TIMEOUT
# 196 None
# 225 None
# 256 None
# TIMEOUT
# 289 None
# 324 None
# TIMEOUT
# 361 None
# TIMEOUT
# 400 None
# TIMEOUT
# 441 None
# TIMEOUT
# 484 None
# 529 None
# 576 None
# 625 None
# 676 None
# 729 None
# TIMEOUT
# 784 None
# 841 None
# STOP None
# 
# ------------------------------- Using processes -------------------------------
# 
# 	######## test_queue2xi
# 
# test_queue2xi: TIMEOUT
# 0 None
# test_queue2xi: TIMEOUT
# 1 None
# 4 None
# 9 None
# 16 None
# 25 None
# 36 None
# 49 None
# 64 None
# 81 None
# test_queue2xi: TIMEOUT
# 100 None
# test_queue2xi: TIMEOUT
# 121 None
# test_queue2xi: TIMEOUT
# 144 None
# test_queue2xi: TIMEOUT
# 169 None
# 196 None
# 225 None
# test_queue2xi: TIMEOUT
# 256 None
# test_queue2xi: TIMEOUT
# 289 None
# test_queue2xi: TIMEOUT
# 324 None
# test_queue2xi: TIMEOUT
# 361 None
# test_queue2xi: TIMEOUT
# 400 None
# test_queue2xi: TIMEOUT
# 441 None
# 484 None
# test_queue2xi: TIMEOUT
# 529 None
# test_queue2xi: TIMEOUT
# 576 None
# 625 None
# 676 None
# 729 None
# 784 None
# test_queue2xi: TIMEOUT
# 841 None
# STOP None    
# 
# ------------------------------- Using processes -------------------------------
# 
# 	######## test_condition
# 
# <Condition(<RLock(None, 0)>, 0)>
# <Condition(<RLock(MainProcess, 1)>, 0)>
# <Condition(<RLock(MainProcess, 2)>, 0)>
# main is waiting
# main has woken up
# <Condition(<RLock(MainProcess, 2)>, 0)>
# <Condition(<RLock(MainProcess, 1)>, 0)>
# <Condition(<RLock(None, 0)>, 0)>
# ------------------------------- Using processes -------------------------------
# 
# 	######## test_condition2xi
# 
# test_condition2xi/cond2xi =  <Condition(<RLock(None, 0)>, 0)>
# test_condition2xi/cond2xi =  <Condition(<RLock(MainProcess, 1)>, 0)>
# test_condition2xi/cond2xi =  <Condition(<RLock(MainProcess, 2)>, 0)>
# test_condition2xi:  main is waiting
# test_condition2xi:  main has woken up
# test_condition2xi/cond2xi =  <Condition(<RLock(MainProcess, 2)>, 0)>
# test_condition2xi/cond2xi =  <Condition(<RLock(MainProcess, 1)>, 0)>
# test_condition2xi/cond2xi =  <Condition(<RLock(None, 0)>, 0)>
#
# ------------------------------- Using processes -------------------------------
# 
# 	######## test_semaphore: ok to console
# 
# ------------------------------- Using processes -------------------------------
# 
# 	######## test_semaphore2xi: ok to console
# 
# 	######## test_join_timeout: ok to console
# 
# waiting for process to finish <--- child sleeping
# . None
# . None
# . None
# . None
# . None <--- child terminating
# ------------------------------- Using processes -------------------------------
# 
# 	######## test_join_timeout2xi ok to console
# 
# test_join_timeout2xi: waiting for process to finish <--- child sleeping
# . None
# . None
# . None
# . None
# . None <--- child terminating
# 
# ------------------------------- Using processes -------------------------------
# 
# 	######## test_event <--- ok to console
# 
# main is sleeping
# main is setting event
# ------------------------------- Using processes -------------------------------
# 
# 	######## test_event2xi <--- ok to console
#
# ------------------------------- Using processes -------------------------------
# 
# 	######## test_sharedvalues
# 
# ------------------------------- Using processes -------------------------------
# 
# 	######## test_sharedvalues2xi 
# -------------------------------------------------------------------------------------------------
# ALL TESTS WITH BOTH STANDARD AND DRAGONIZED MULTIPROCESSING                               
# note: console output was ok
# -------------------------------------------------------------------------------------------------
# ------------------------------- Using processes -------------------------------
# 
# 	######## test_value
# 
# 10 None
# 10 None
# 10 None
# 10 None
# 10 None
# 10 None
# 10 None
# 10 None
# 10 None
# 10 None
# 10 None
# 10 None
# 10 None
# 10 None
# 10 None
# 10 None
# 10 None
# 10 None
# 10 None
# 9 None
# 9 None
# 9 None
# 9 None
# 8 None
# 8 None
# 7 None
# 7 None
# 7 None
# 7 None
# 7 None
# 7 None
# 7 None
# 7 None
# 6 None
# 5 None
# 4 None
# 3 None
# 3 None
# 3 None
# 3 None
# 3 None
# 3 None
# 2 None
# 2 None
# 2 None
# 1 None
# 0 None
# No more running processes
# 
# 	######## test_queue
# 
# TIMEOUT
# 0 None
# 1 None
# 4 None
# 9 None
# 16 None
# 25 None
# 36 None
# TIMEOUT
# 49 None
# 64 None
# 81 None
# 100 None
# TIMEOUT
# 121 None
# 144 None
# TIMEOUT
# 169 None
# TIMEOUT
# 196 None
# TIMEOUT
# 225 None
# 256 None
# TIMEOUT
# 289 None
# TIMEOUT
# 324 None
# TIMEOUT
# 361 None
# TIMEOUT
# 400 None
# 441 None
# 484 None
# 529 None
# TIMEOUT
# 576 None
# TIMEOUT
# 625 None
# 676 None
# 729 None
# 784 None
# 841 None
# STOP None
# 
# 
# 	######## test_condition
# 
# <Condition(<RLock(None, 0)>, 0)>
# <Condition(<RLock(MainProcess, 1)>, 0)>
# <Condition(<RLock(MainProcess, 2)>, 0)>
# main is waiting
# main has woken up
# <Condition(<RLock(MainProcess, 2)>, 0)>
# <Condition(<RLock(MainProcess, 1)>, 0)>
# <Condition(<RLock(None, 0)>, 0)>
# 
# 	######## test_semaphore
# 
# 
# 	######## test_join_timeout
# 
# waiting for process to finish
# . None
# . None
# . None
# . None
# . None
# 
# 	######## test_event
# 
# main is sleeping
# main is setting event
# 
# 	######## test_sharedvalues
# 
# ------------------------------- Using processes -------------------------------
# 
# 	######## test_value2xi
# 
# 10 None
# 10 None
# 10 None
# 10 None
# 10 None
# 10 None
# 10 None
# 10 None
# 10 None
# 10 None
# 10 None
# 10 None
# 10 None
# 9 None
# 9 None
# 9 None
# 9 None
# 8 None
# 8 None
# 8 None
# 8 None
# 8 None
# 8 None
# 8 None
# 8 None
# 8 None
# 8 None
# 8 None
# 7 None
# 7 None
# 7 None
# 7 None
# 5 None
# 4 None
# 3 None
# 3 None
# 3 None
# 2 None
# 2 None
# 2 None
# 2 None
# 2 None
# 2 None
# 1 None
# 0 None
# test_value2xi: No more running processes
# 
# 	######## test_queue2xi
# 
# test_queue2xi: TIMEOUT
# test_queue2xi: TIMEOUT
# 0 None
# test_queue2xi: TIMEOUT
# 1 None
# test_queue2xi: TIMEOUT
# 4 None
# test_queue2xi: TIMEOUT
# 9 None
# 16 None
# test_queue2xi: TIMEOUT
# 25 None
# 36 None
# test_queue2xi: TIMEOUT
# 49 None
# 64 None
# 81 None
# 100 None
# test_queue2xi: TIMEOUT
# 121 None
# test_queue2xi: TIMEOUT
# 144 None
# 169 None
# 196 None
# 225 None
# 256 None
# test_queue2xi: TIMEOUT
# 289 None
# 324 None
# test_queue2xi: TIMEOUT
# 361 None
# test_queue2xi: TIMEOUT
# 400 None
# 441 None
# test_queue2xi: TIMEOUT
# 484 None
# 529 None
# 576 None
# 625 None
# 676 None
# test_queue2xi: TIMEOUT
# 729 None
# test_queue2xi: TIMEOUT
# 784 None
# test_queue2xi: TIMEOUT
# 841 None
# STOP None
# 
# 
# 	######## test_condition2xi
# 
# test_condition2xi/cond2xi =  <Condition(<RLock(None, 0)>, 0)>
# test_condition2xi/cond2xi =  <Condition(<RLock(MainProcess, 1)>, 0)>
# test_condition2xi/cond2xi =  <Condition(<RLock(MainProcess, 2)>, 0)>
# test_condition2xi:  main is waiting
# test_condition2xi:  main has woken up
# test_condition2xi/cond2xi =  <Condition(<RLock(MainProcess, 2)>, 0)>
# test_condition2xi/cond2xi =  <Condition(<RLock(MainProcess, 1)>, 0)>
# test_condition2xi/cond2xi =  <Condition(<RLock(None, 0)>, 0)>
# 
# 	######## test_semaphore2xi
# 
# 
# 	######## test_join_timeout2xi
# 
# test_join_timeout2xi: waiting for process to finish
# . None
# . None
# . None
# . None
# . None
# 
# 	######## test_event2xi
# 
# test_event2xi: main is sleeping
# test_event2xi: main is setting event
# 
# 	######## test_sharedvalues2xi
# 
# -------------------------------------------------------------------------------------------------
# End Of test_locks_mp_xi2.py                               
# -------------------------------------------------------------------------------------------------